package va.montecarlo;

import java.io.*;
import java.lang.reflect.Constructor;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import va.curve.MortalityCurve;
import va.policy.Inforce;
import va.policy.Policy;
import va.scenario.FundMap;
import va.scenario.Scenario;

public class PriceSoSHelper {
	protected Logger log = Logger.getLogger(PriceSoSHelper.class);
	protected Map<String, String> mapParameters;
	protected int startRecordID;
	protected int endRecordID;
	protected String inforceFile;
	protected String RNFolder;
	protected String RWFolder;
	protected int startRWPath;
	protected int endRWPath;
	protected List<Integer> lstRWValuationTime;
	
	protected int numIndex;
	protected int numScenario;
	protected int numTimeStep;
	protected FundMap fundMap;
	
	protected double[] vFC;
	protected MortalityCurve mcfemale;
	protected MortalityCurve mcmale;
	protected Scenario scenarioRN;
	protected Scenario scenarioRW;
	
	protected Inforce inforce;
	protected List<String> listErrorMsg;
	protected Map<Integer, double[][]> mapResult;
	protected double[][] mPayoff;
		
	protected String paramFile;
	
	protected String aggregateOutputFile;
	protected String seriatimOutputFile;
	protected double totalRunTime;
	protected String operation;
	protected int numThread;
	
	protected int processingPath;
	protected Map<Integer, Double> shockU;
	protected Map<Integer, Double> shockD;
	
	String newline = System.getProperty("line.separator");
	
	public PriceSoSHelper(String paramFile, String operation) {
		this.paramFile = paramFile;		
		this.operation = operation.trim();
		listErrorMsg = new ArrayList<String>();
		mapResult = new HashMap<Integer, double[][]>();		
	}
	
	public void price() throws Exception {
		loadParameter();
		loadInforce();
		
		if(operation.equals("value")) {
			double[] vShock = new double[numIndex];
			Arrays.fill(vShock, 0.01);
			shockU = fundMap.convert(vShock);
			
			Arrays.fill(vShock, -0.01);
			shockD = fundMap.convert(vShock);
			

			loadMortalityCurve();
			loadScenario();			
			doWork();			
		} else if(operation.equals("payoff") ){
			loadMortalityCurve();
			loadScenario();	
			calPayoff();
		} else if(operation.equals("createrunfile")) {			
			StringBuilder sb = new StringBuilder();
			sb.append("rwpath,status,runtime");		
			sb.append(newline);
			for(int k=startRWPath; k<=endRWPath; ++k) {
				sb.append(String.format("%d,0,%s", k, newline));
				// 0 - new, 1- running, 2-finished, 3-error
			}
			
			PrintWriter out = new PrintWriter(getParam("IVRunFile"));
			out.write(sb.toString());
			out.close();
		} else {
			throw new Exception("unknown operation: " + operation);
		}
	}
	
	protected void doWork() throws Exception {
		while(true) {
			RandomAccessFile file = null;
			FileLock fileLock = null;
			processingPath =0;	        
	        totalRunTime = 0;
	        try {	        	
	        	String IVRunFile = getParam("IVRunFile");
	        	String lockFile = IVRunFile.replaceFirst("[.][^.]+$", "") + ".lock";
	        	file = new RandomAccessFile(lockFile, "rw");
	        	FileChannel fileChannel = file.getChannel();
	        	while(true) {
	        		fileLock = fileChannel.tryLock();
	        		if(fileLock!=null) {
	        			break;
	        		}
	        		Thread.sleep(1000);
	        	}
	        	
	        	StringBuilder sb = new StringBuilder();
	        	BufferedReader br = new BufferedReader(new FileReader(IVRunFile));
	        	String[] cell;
	        	sb.append(br.readLine()).append(newline);
	        	while(true){
	        		String line = br.readLine();
	        		if(line == null || line.trim().equals("")) {
	        			break;
	        		}
	        	    cell = line.split(",");
	        		if(Integer.parseInt(cell[1])==0 && processingPath==0) {
	        			processingPath = Integer.parseInt(cell[0]);	        			
	        			sb.append(String.format("%s,1,%s", cell[0], newline));
	        		} else {
	        			sb.append(line);
	        			sb.append(newline);
	        		}
	        	}
	        	br.close();	
	        	
	        	FileWriter outFile = new FileWriter(IVRunFile);
	    		PrintWriter out = new PrintWriter(outFile);
	    		out.print(sb.toString());
	    		out.close();
	        	
	        } finally {	
	        	if(fileLock != null) {
	        		fileLock.release();
	        	}
	        	file.close();
	        }	 
	        
	        if(processingPath==0) {
	        	break;
	        }
	        	        
	        Logger log = Logger.getRootLogger();	        
			log.info(String.format("Processing path %d using %d threads", processingPath, numThread));
								       
			int nLen = (int) Math.floor(inforce.size() / numThread);
			int nRem = inforce.size()-nLen*numThread;
			List<Thread> listThread = new ArrayList<Thread>();
			
			int end = -1;
			for(int k=0; k<numThread; ++k) {
				int beg = end + 1;
				end = beg + nLen - 1;
				if(nRem >0) {
					end++;
					nRem--;
				}
				if(end < beg) {
					break;
				}
				
				class MyTask implements Runnable {
					private int a;
					private int b;
					public MyTask(int a, int b) {
						this.a = a;
						this.b = b;
					}
				    public void run(){
				       doWork(a,b);
				    }				  
				}
				
				Thread t = new Thread(new MyTask(beg, end));
				listThread.add(t);
				t.start();
			}
            
			for(Thread t: listThread) {
				t.join();
			}
			
            log.info(String.format("Runtime used for %d: %f", processingPath, totalRunTime));
			saveResult();	        
	        
			try {	        	
	        	String IVRunFile = getParam("IVRunFile");
	        	String lockFile = IVRunFile.replaceFirst("[.][^.]+$", "") + ".lock";
	        	file = new RandomAccessFile(lockFile, "rw");
	        	FileChannel fileChannel = file.getChannel();
	        	while(true) {
	        		fileLock = fileChannel.tryLock();
	        		if(fileLock!=null) {
	        			break;
	        		}
	        		Thread.sleep(1000);
	        	}
	        	
	        	StringBuilder sb = new StringBuilder();
	        	BufferedReader br = new BufferedReader(new FileReader(IVRunFile));
	        	String line = br.readLine();
	        	sb.append(line).append(newline);
	        	while(true) {
	        		line = br.readLine();
	        		if(line == null || line.trim().equals("")) {
	        			break;
	        		}
	        		
	        		String[] cell = line.split(",");
	        		if(Integer.parseInt(cell[0])==processingPath) {	
	        			sb.append(String.format("%s,2,%f", cell[0],totalRunTime));
	        			sb.append(newline);
	        		} else {
	        			sb.append(line).append(newline);
	        		}
	        	}
	        	br.close();
	        	
	        	FileWriter outFile = new FileWriter(IVRunFile);
	    		PrintWriter out = new PrintWriter(outFile);
	    		out.print(sb.toString());
	    		out.close();
	        } finally {	        	
	        	if(fileLock != null) {
	        		fileLock.release();
	        	}
	        	file.close();
	        }
		}
	}
	
	
	protected void doWork(int beg, int end) {
		long start = System.currentTimeMillis();
		
		Map<Integer, double[][]> mapLocalResult = new HashMap<Integer, double[][]>();
		
		for(int i=beg; i<=end; ++i) {
			Policy p = inforce.get(i);			
			double[] vDelta = new double[lstRWValuationTime.size()];
			Arrays.fill(vDelta, 0.0);
			int numIndex = fundMap.getNumIndex();
			double[][] mDelta = new double[numIndex][lstRWValuationTime.size()];
			for(int k=0; k<numIndex; ++k) {
				Arrays.fill(mDelta[k], 0.0);
			}
						
			for(int j=0; j<lstRWValuationTime.size(); ++j) {
				int scenarioOffset = lstRWValuationTime.get(j)-1;
				Policy pj = p.clone();
				Pricer ager = getAger(p.productType);
				if(ager == null) {
					break;
				}
				ager.setValue(vFC, mcfemale, mcmale);
				for(int t=0; t<scenarioOffset; ++t) {
					ager.project(pj, processingPath-1, t);
				}
				double[] av = pj.getAVbyIndex(fundMap);
				if(av[numIndex] < 1e-4) {
					break;
				}
				
				if(scenarioOffset < 0) {
					scenarioOffset = 0;
				}
				// fmv_u
				Pricer pricer = getPricer(p.productType, scenarioOffset);
				Result res = pricer.price(pj, shockU, vFC, mcfemale, mcmale, fundMap);
				double fmvU = res.fmv;
				//*
				if(p.recordID==10 && processingPath==1 && j==29) {
					log.info(String.format("%s, %s", p.currentDate.toString(), p.matDate.toString()));
					String postfix = "temp";
					try{
						pricer.saveAV(postfix);
						pricer.saveDA(postfix);
						pricer.saveLA(postfix);
						pricer.saveRC(postfix);
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
				//*/
				// fmv_d
				pricer = getPricer(p.productType, scenarioOffset);
				res = pricer.price(pj, shockD, vFC, mcfemale, mcmale, fundMap);
				double fmvD = res.fmv;
				
				vDelta[j] = (fmvU - fmvD) / 0.02;
				for(int k=0; k<numIndex; ++k) {
					mDelta[k][j] = vDelta[j] * av[k] / av[numIndex];
				}
			}						
			
			mapLocalResult.put(p.recordID, mDelta);
		}
		double timeElapsed = (System.currentTimeMillis() - start) / 1000.0;
		
		synchronized(this) {
			mapResult.putAll(mapLocalResult);
			totalRunTime += timeElapsed;			
		}
	}
	
	protected void calPayoff() throws Exception {
		mPayoff = new double[numScenario][numTimeStep];
		for(int i=0; i<numScenario; ++i) {
		    Arrays.fill(mPayoff[i], 0.0);
		}
		double[] vDF = new double[numTimeStep];

		for(int i=0; i<inforce.size(); ++i) {
			Policy p = inforce.get(i);
			
			double[] vShock = new double[numIndex];
			Arrays.fill(vShock, 0.0);
			Map<Integer, Double> mShock = fundMap.convert(vShock);
			Pricer pricer = getAger(p.productType);
			pricer.price(p, mShock, vFC, mcfemale, mcmale, fundMap);
			double[][] mDA = pricer.getDA();
			double[][] mLA = pricer.getLA();
			//double[][] mRC = pricer.getRC();
			for(int s=0; s<numScenario; ++s) {
				for(int t=0; t<numTimeStep; ++t) {
					mPayoff[s][t] += mDA[s][t] + mLA[s][t]; // - mRC[s][t];
				}
			}
			
			if(i==0) {
				double[] vTmp = pricer.getDF();
				for(int t=0; t<numTimeStep; ++t) {
					vDF[t] = vTmp[t+1];
				}
			}
			
			if((i+1) % 100 == 0) {
				log.info(String.format("Processed %d policies", i+1));
			}
		}
		
		StringBuilder sb = new StringBuilder();
		sb.append("rwpath");
		for(int j=0; j<numTimeStep; ++j) {
			sb.append(",T").append(j);
		}			
		sb.append(newline);
		sb.append("0");
		for(int j=0; j<numTimeStep; ++j) {
			sb.append(String.format(",%.6f", vDF[j]));
		}			
		sb.append(newline);
		for(int s=0; s<numScenario; ++s) {
			sb.append(s+1);
			for(int t=0; t<numTimeStep; ++t) {
				sb.append(String.format(",%.6f", mPayoff[s][t]));
			}
			sb.append(newline);
		}
		
		String outputFile = "payoff.csv";
		FileWriter outFile = new FileWriter(outputFile);
		PrintWriter out = new PrintWriter(outFile);
		out.print(sb.toString());
		out.close();
	}
	
	protected Pricer getAger(String productType) {
		String className = "va.montecarlo.Pricer" + productType;
		Pricer pricer = null;
		try{
			Constructor<?> cons = Class.forName(className).getConstructor(Scenario.class, int.class); 
			pricer = (Pricer) cons.newInstance(scenarioRW, 0);
		}catch (Exception ex) {
			log.error("Cannot find pricer for " + productType);
		}
				
		return pricer;
	}
	
	protected Pricer getPricer(String productType, int scenarioOffset) {
		String className = "va.montecarlo.Pricer" + productType;
		Pricer pricer = null;
		try{
			Constructor<?> cons = Class.forName(className).getConstructor(Scenario.class, int.class); 
			pricer = (Pricer) cons.newInstance(scenarioRN, scenarioOffset);
		}catch (Exception ex) {
			log.error("Cannot find pricer for " + productType);
		}
				
		return pricer;
	}
	
	protected void saveResult() throws Exception {
		String newline = System.getProperty("line.separator");
		int numIndex = fundMap.getNumIndex();
		if(seriatimOutputFile.length() >0 ) {
			for(int k=0; k<numIndex; ++k ) {
				StringBuilder sb = new StringBuilder();
				sb.append("RecordID");
				for(int j=0; j<lstRWValuationTime.size(); ++j) {
					sb.append(",T").append(lstRWValuationTime.get(j));
				}			
				sb.append(newline);
				for(Integer id : mapResult.keySet()) {
					sb.append(id);
					double[][] vTmp = mapResult.get(id);
					for(int j=0; j<lstRWValuationTime.size(); ++j) {
					    sb.append(String.format(",%.6f",vTmp[k][j]));				
					}
				    sb.append(newline);
				}
				
				String outputFile = String.format("%s_%d_%d.csv", 
						seriatimOutputFile.replaceFirst("[.][^.]+$", ""), processingPath, k+1);
				FileWriter outFile = new FileWriter(outputFile);
				PrintWriter out = new PrintWriter(outFile);
				out.print(sb.toString());
				out.close();
			}
		}
		
		if(aggregateOutputFile.length() > 0) {
			for(int k=0; k<numIndex; ++k) {
				double[] vSum = new double[lstRWValuationTime.size()];
				Arrays.fill(vSum, 0.0);
				
				for(Integer id : mapResult.keySet()) {
					double[][] vTmp = mapResult.get(id);
					for(int j=0; j<lstRWValuationTime.size(); ++j) {
						vSum[j] += vTmp[k][j];
					}
				}
				
				StringBuilder sb = new StringBuilder();
				sb.append("rwpath");
				for(int j=0; j<lstRWValuationTime.size(); ++j) {
					sb.append(",T").append(lstRWValuationTime.get(j));
				}			
				sb.append(newline);
	
				sb.append(processingPath);
				for(int j=0; j<vSum.length; ++j) {
				    sb.append(String.format(",%.6f",vSum[j]));				
				}
			    sb.append(newline);
				
				String outputFile = String.format("%s_%d_%d.csv", 
						aggregateOutputFile.replaceFirst("[.][^.]+$", ""), processingPath, k+1);
				FileWriter outFile = new FileWriter(outputFile);
				PrintWriter out = new PrintWriter(outFile);
				out.print(sb.toString());
				out.close();
			}
		}
	}
	
	protected void loadInforce() throws Exception {
		inforce = new Inforce();
		BufferedReader br = new BufferedReader(new FileReader(inforceFile));
		String[] name;
		String[] cell;
		String header = br.readLine();
		name = header.split(",");
		while(true) {
			String line = br.readLine();
			if(line == null || line.trim().equals("")) {
				break;
			}
			cell = line.split(",");
			Policy p = new Policy(name, cell);
			if(p.recordID >= startRecordID && p.recordID <= endRecordID) {
				inforce.addPolicy(p);
			}
		}
		br.close();
	}
	
	protected void loadMortalityCurve() throws Exception {
		mcfemale = new MortalityCurve(getParam("FemaleMortalityFile"));
		mcmale = new MortalityCurve(getParam("MaleMortalityFile"));
	}
	
	protected void loadScenario() throws Exception {
		File folder = new File(RNFolder);
		File file = new File(folder, getParam("CurveFile"));
		
		vFC = new double[numTimeStep];
		BufferedReader br = new BufferedReader(new FileReader(file));
		br.readLine();
		br.readLine();
		for(int i=0; i<numTimeStep; ++i) {				
			vFC[i] = Double.parseDouble(br.readLine());
		}
		br.close();
					
		scenarioRN = new Scenario(RNFolder, fundMap, numScenario, numTimeStep);
		scenarioRW = new Scenario(RWFolder, fundMap, numScenario, numTimeStep);		
	}
	
	protected void loadParameter() throws Exception {
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
        Document doc = docBuilder.parse (new File(paramFile));
        
        doc.getDocumentElement().normalize();
        NodeList nl = doc.getElementsByTagName("Parameters");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of Parameters tag != 1");
        }
        mapParameters = new HashMap<String, String>();
        NodeList nl2 = ((Element)nl.item(0)).getElementsByTagName("Parameter");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name = node.getAttributes().getNamedItem("name").getNodeValue();
        	String value = node.getAttributes().getNamedItem("value").getNodeValue();
        	
        	mapParameters.put(name.toLowerCase(), value);
        }
        
        RNFolder = getParam("RNFolder");
        RWFolder = getParam("RWFolder");
        
        aggregateOutputFile = getParam("AggregateOutputFile").trim();
		seriatimOutputFile = getParam("SeriatimOutputFile").trim();
		
        startRecordID = getIntParam("startRecordID");
        endRecordID = getIntParam("endRecordID");
        inforceFile = getParam("inforceFile");
        
        startRWPath = getIntParam("startRWPath");
        endRWPath = getIntParam("endRWPath");
        
        numIndex = getIntParam("numIndex");
        numScenario = getIntParam("numScenario");
        numTimeStep = getIntParam("numTimeStep");
        numThread = getIntParam("numThread");
        fundMap = new FundMap(getParam("fundMapFile"));
       
        String sValuationTime = getParam("RWValuationTime");
        String[] cell = sValuationTime.split(",");
        lstRWValuationTime = new ArrayList<Integer>();
        for(int i=0; i<cell.length; ++i) {
        	lstRWValuationTime.add(Integer.parseInt(cell[i]));
        }
	}
	
	protected int getIntParam(String name) throws Exception {
		String key = name.toLowerCase();
		if(mapParameters.containsKey(key)) {
			return Integer.parseInt(mapParameters.get(key));
		} else {
			throw new Exception("Cannot find parameter: " + name);
		}
	}
	
	protected String getParam(String name) throws Exception {
		String key = name.toLowerCase();
		if(mapParameters.containsKey(key)) {
			return mapParameters.get(key);
		} else {
			throw new Exception("Cannot find parameter: " + name);
		}
	}
}
