package va.montecarlo;

import va.calendar.SDate;
import va.policy.Policy;
import va.scenario.Scenario;

public class PricerDBSU extends Pricer {
	
	public PricerDBSU(Scenario scenario, int scenarioOffset) {
		super(scenario, scenarioOffset);
		this.name = "DBSU Pricer";
	}
	
	@Override
	public void project(Policy p, int sceInd, int timeInd) {
		int T = SDate.monthsBetween(p.currentDate, p.matDate); 		
		
		if(T<0) { // maturity
			AV[sceInd][timeInd] = 0.0;
			DA[sceInd][timeInd] = 0.0;
			LA[sceInd][timeInd] = 0.0;
			RC[sceInd][timeInd] = 0.0;
			
			return;
		}
		
		int nMonth = SDate.monthsBetween(p.issueDate, p.currentDate);
		
		int numFund = p.vFundNum.length;
		double dAV = 0.0;
		double dFee = 0.0;
		for(int k=0; k<numFund; ++k) {					
			double dPartialAV = p.vFundValue[k] *  
					scenario.getFundScenario(p.vFundNum[k])[sceInd][timeInd+scenarioOffset] *
			        (1 - p.vFundFee[k] * dT);
			double dPartialFee = 0.0;
			double dBaseFee = 0.0;
			if(nMonth % 12 == 0  && nMonth >0) {
			    dPartialFee = dPartialAV * p.riderFee;
			    dBaseFee = dPartialAV * p.baseFee;
			}
			p.vFundValue[k] = dPartialAV - dPartialFee - dBaseFee;			
			dAV += p.vFundValue[k] ;		
			dFee += dPartialFee;				
		}
		
		// update the policy information
		p.currentDate.addMonths(1);	
		if(nMonth % 12 == 0  && nMonth >0) {
		    p.gbAmt = Math.max(dAV, p.gbAmt);
		}		
					
		AV[sceInd][timeInd] = dAV;
		DA[sceInd][timeInd] = Math.max(0, p.gbAmt-dAV);
		DA[sceInd][timeInd] = 0.0;
		RC[sceInd][timeInd] = dFee;			
		
	}

}
