package va.montecarlo;

import va.calendar.SDate;
import va.policy.Policy;
import va.scenario.Scenario;

public class PricerWBRP extends Pricer {
	
	public PricerWBRP(Scenario scenario, int scenarioOffset) {
		super(scenario, scenarioOffset);
		this.name = "WBRP Pricer";
	}

	@Override
	public void project(Policy p, int sceInd, int timeInd) {	
		int T = SDate.monthsBetween(p.currentDate, p.matDate); // (int) ((p.matDate.year() - p.currentDate.year() ) / dT);		
		
		if(T<0) { // maturity
			AV[sceInd][timeInd] = 0.0;
			DA[sceInd][timeInd] = 0.0;
			LA[sceInd][timeInd] = 0.0;
			RC[sceInd][timeInd] = 0.0;
			
			return;
		}
		
		int nMonth =  SDate.monthsBetween(p.issueDate, p.currentDate);
		
		int numFund = p.vFundNum.length;
		double dAV = 0.0;
		double dFee = 0.0;		
		for(int k=0; k<numFund; ++k) {					
			double dPartialAV = p.vFundValue[k] *  
					scenario.getFundScenario(p.vFundNum[k])[sceInd][timeInd+scenarioOffset] *
			        (1 - p.vFundFee[k] * dT);
			double dPartialFee = 0.0;
			double dBaseFee = 0.0;
			if(nMonth % 12 == 0  && nMonth >0) {
			    dPartialFee = dPartialAV * p.riderFee;
			    dBaseFee = dPartialAV * p.baseFee;
			}
			
			p.vFundValue[k] = dPartialAV - dPartialFee - dBaseFee;			
			dAV += p.vFundValue[k] ;		
			dFee += dPartialFee;
		}
		
		double dWA = 0.0;
		if(nMonth % 12 == 0  && nMonth >0) {
			double dWAG = p.gbAmt * p.wbWithdrawalRate;
			dWA = Math.min(dWAG, p.gmwbBalance);
			p.gmwbBalance -= dWA;
			p.withdrawal += dWA;
			
			if(p.gmwbBalance < 1e-3) {
				p.gbAmt = 0;
			}
			
			dAV = Math.max(0.0, dAV - dWA);
			if(dAV > 1e-4) {
				for(int k=0; k<numFund; ++k) {		
					p.vFundValue[k] *= dAV / (dAV+dWA);
				}		
			} else {
				for(int k=0; k<numFund; ++k) {		
					p.vFundValue[k] = 0.0;
				}
			}
		}
		
		// update the policy information
		p.currentDate.addMonths(1);		
					
		AV[sceInd][timeInd] = dAV;
		DA[sceInd][timeInd] = 0.0;
		LA[sceInd][timeInd] = Math.max(0.0, dWA - dAV);
		RC[sceInd][timeInd] = dFee;		
		if(T == 0) {
			LA[sceInd][timeInd] = Math.max(0.0, dWA + p.gmwbBalance - dAV);
		} else {
			LA[sceInd][timeInd] = Math.max(0.0, dWA - dAV);
		}
		// the maturity date is after the last scenario date
		if(T > 0 && numTimeStep -1 == timeInd + scenarioOffset) {
			LA[sceInd][timeInd] = Math.max(0.0, dWA + p.gmwbBalance - dAV);
		}
	}

}
