package va.policy;

import java.util.*;

import va.calendar.SDate;
import va.scenario.FundMap;

public class Policy {
	public int recordID;
	public double survivorShip;
	public String gender; // M, F
	public String productType;
	public SDate issueDate;
	public SDate matDate; // maturity date
	public SDate birthDate;
	public SDate currentDate; // this policy is aged to this date
	public double baseFee; //MER
	public double riderFee; // 	
	public double rollUpRate;
	public double gbAmt = 0.0; // guaranteed benefit
	public double gmwbBalance = 0.0;
	public double wbWithdrawalRate = 0.0;	
	public double withdrawal = 0.0; // withdrawal so far
	
	// fund information
	public int[] vFundNum;
	public double[] vFundValue;
	public double[] vFundFee;
	
	public Policy() {
	}
	
	public Policy clone() {
		Policy res = new Policy();
		res.recordID = recordID;
		res.survivorShip = survivorShip;
		res.gender = gender;
		res.productType = productType;
		res.issueDate = new SDate(issueDate);
		res.matDate = new SDate(matDate);
		res.birthDate = new SDate(birthDate);
		res.currentDate = new SDate(currentDate);
		res.baseFee = baseFee;
		res.riderFee = riderFee;
		res.rollUpRate = rollUpRate;
		res.gbAmt = gbAmt;
		res.gmwbBalance = gmwbBalance;
		res.wbWithdrawalRate = wbWithdrawalRate;
		res.withdrawal = withdrawal;
		
		res.vFundNum = vFundNum.clone();
		res.vFundValue = vFundValue.clone();
		res.vFundFee = vFundFee.clone();
		
		return res;
	}
			
	public Policy(String[] name, String[] value) {
		Map<String, String> mapValue = new HashMap<String, String>();
		for(int i=0; i<name.length; ++i) {
			mapValue.put(name[i].trim(), value[i]);
		}
		recordID = Integer.parseInt(mapValue.get("recordID"));		
		survivorShip = Double.parseDouble(mapValue.get("survivorShip"));
		gender = mapValue.get("gender");
		productType = mapValue.get("productType");
		issueDate = new SDate();
		issueDate.setDateExcel(Double.parseDouble(mapValue.get("issueDate")));
		matDate = new SDate();
		matDate.setDateExcel(Double.parseDouble(mapValue.get("matDate")));
		birthDate = new SDate();
		birthDate.setDateExcel(Double.parseDouble(mapValue.get("birthDate")));
		currentDate = new SDate();
		currentDate.setDateExcel(Double.parseDouble(mapValue.get("currentDate")));
		baseFee = Double.parseDouble(mapValue.get("baseFee"));
		riderFee = Double.parseDouble(mapValue.get("riderFee"));
		rollUpRate = Double.parseDouble(mapValue.get("rollUpRate"));
		gbAmt = Double.parseDouble(mapValue.get("gbAmt"));
		gmwbBalance = Double.parseDouble(mapValue.get("gmwbBalance"));
		wbWithdrawalRate = Double.parseDouble(mapValue.get("wbWithdrawalRate"));
		withdrawal = Double.parseDouble(mapValue.get("withdrawal"));
		int numFund = 0;
		for(String key : mapValue.keySet()) {
			if(key.startsWith("FundNum")) {
				numFund++;
			}
		}
		vFundNum = new int[numFund];
		vFundValue = new double[numFund];
		vFundFee = new double[numFund];
		for(int j=0; j<numFund; ++j) {
			vFundNum[j] = Integer.parseInt(mapValue.get(String.format("FundNum%d", j+1)));
			vFundValue[j] = Double.parseDouble(mapValue.get(String.format("FundValue%d", j+1)));
			vFundFee[j] = Double.parseDouble(mapValue.get(String.format("FundFee%d", j+1)));
		}
	}
	
	public String getHeader() {
		StringBuilder sb = new StringBuilder();
		sb.append("recordID,").append("survivorShip,").append("gender,").append("productType,").append("issueDate,");
		sb.append("matDate,").append("birthDate,").append("currentDate,").append("baseFee,");
		sb.append("riderFee,").append("rollUpRate,").append("gbAmt,");
		sb.append("gmwbBalance,").append("wbWithdrawalRate,").append("withdrawal,");
	
		for(int j=0; j<vFundNum.length; ++j) {
			sb.append(String.format("FundValue%d,", j+1));
		}	
		for(int j=0; j<vFundNum.length; ++j) {
			sb.append(String.format("FundNum%d,", j+1));
		}
		for(int j=0; j<vFundNum.length-1; ++j) {
			sb.append(String.format("FundFee%d,", j+1));
		}
		sb.append(String.format("FundFee%d", vFundNum.length));
		
		return sb.toString();
	}
	
	public String getString() {		
		StringBuilder sb = new StringBuilder();		
		sb.append(String.format("%d,%.6f,%s,%s,%.0f,", recordID, survivorShip, gender,productType,issueDate.asExcelDate()));
		if(currentDate == null) {
			sb.append(String.format("%.0f,%.0f,%.0f,", matDate.asExcelDate(), birthDate.asExcelDate(),
					0.0));
		} else {
			sb.append(String.format("%.0f,%.0f,%.0f,", matDate.asExcelDate(), birthDate.asExcelDate(),
					currentDate.asExcelDate()));
		}
		sb.append(String.format("%.6f,%.6f,%.6f,%.6f,", baseFee, riderFee, rollUpRate,gbAmt));
		sb.append(String.format("%.6f,%.6f,%.6f,", gmwbBalance, wbWithdrawalRate, withdrawal));
	
		for(int j=0; j<vFundNum.length; ++j) {
			sb.append(String.format("%.6f,", vFundValue[j]));
		}	
		for(int j=0; j<vFundNum.length; ++j) {
			sb.append(String.format("%d,", vFundNum[j]));
		}
		for(int j=0; j<vFundNum.length-1; ++j) {
			sb.append(String.format("%.6f,", vFundFee[j]));
		}
		sb.append(String.format("%.6f", vFundFee[vFundNum.length-1]));
		
		return sb.toString();
	
	}			
		
	public double[] getAVbyIndex(FundMap fundMap) {
		int numIndex = fundMap.getNumIndex();
		double[] av = new double[numIndex + 1];
		av[numIndex] = 0.0;
		for(int i=0; i<numIndex; ++i) {
			double dSum = 0.0;
			for(int k=0; k<vFundNum.length; ++k) {
				dSum += vFundValue[k]  * fundMap.get(vFundNum[k])[i];
			}
			av[i] = dSum;
			av[numIndex] += dSum;
		}
		
		return av;
	}
}
