package va.scenario;

import java.io.*;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.RealMatrix;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class GeneratorRSHelper {
	private String paramFile;
	
	private Map<String, String> mapParameters;
	private int numScenario;
	private int numYear;
	private int numStep;
	private String outputFolder;
	private int seed;
	private double dT;
	private List<String> listIndexName;
	private List<String> listIndexFileName;
	private Map<String, Double> mapCor;
	private Map<String, double[]> mapVol;
	private Map<String, double[]> mapMu;
	private double dP12;
	private double dP21;
	
	
	public GeneratorRSHelper(String paramFile) {
		this.paramFile = paramFile;
		
	}
	public void generate() throws Exception {
		loadParameter();
		
		int numIndex = listIndexName.size();
		RealMatrix mCov1 = new Array2DRowRealMatrix(numIndex, numIndex);
		RealMatrix mCov2 = new Array2DRowRealMatrix(numIndex, numIndex);
		double[] vMu1 = new double[numIndex];
		double[] vMu2 = new double[numIndex];
		for(int i=0; i<numIndex; ++i) {
			vMu1[i] = getMu(listIndexName.get(i), 0);
			vMu2[i] = getMu(listIndexName.get(i), 1);
			
			for(int j=0; j<i; ++j) {
				double dCov = getCor(listIndexName.get(i), listIndexName.get(j)) * 
						getVol(listIndexName.get(i), 0) * getVol(listIndexName.get(j), 0);
				mCov1.setEntry(i, j, dCov);
				mCov1.setEntry(j, i, dCov);
			}
			mCov1.setEntry(i, i, getVol(listIndexName.get(i), 0) * getVol(listIndexName.get(i), 0));
		 
			for(int j=0; j<i; ++j) {
				double dCov = getCor(listIndexName.get(i), listIndexName.get(j)) * 
						getVol(listIndexName.get(i), 1) * getVol(listIndexName.get(j), 1);
				mCov2.setEntry(i, j, dCov);
				mCov2.setEntry(j, i, dCov);
			}
			mCov2.setEntry(i, i, getVol(listIndexName.get(i), 1) * getVol(listIndexName.get(i), 1));
		}
		
		
		GeneratorRS g = new GeneratorRS(listIndexFileName, mCov1, mCov2, vMu1, vMu2, dP12, dP21,  
				seed, numScenario, numStep, dT);
		g.generate();
		g.save(outputFolder);
		
	}
	
	protected void loadParameter() throws Exception {
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
        Document doc = docBuilder.parse (new File(paramFile));
        
        doc.getDocumentElement().normalize();
        NodeList nl = doc.getElementsByTagName("Parameters");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of Parameters tag != 1");
        }
        
        mapParameters = new HashMap<String, String>();
        NodeList nl2 = ((Element)nl.item(0)).getElementsByTagName("parameter");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name = node.getAttributes().getNamedItem("name").getNodeValue();
        	String value = node.getAttributes().getNamedItem("value").getNodeValue();
        	
        	mapParameters.put(name.toLowerCase(), value);
        }
        
        numScenario = getIntParam("numScenario");     
        numStep = getIntParam("numStep");  
        numYear = getIntParam("numYear");  
        dT = (double) numYear / numStep;
        outputFolder = getParam("outputFolder");
        seed = getIntParam("seed");  
        dP12 = getDoubleParam("p12");
        dP21 = getDoubleParam("p21");
        
        listIndexName = new ArrayList<String>();
        listIndexFileName = new ArrayList<String>();
        nl = doc.getElementsByTagName("Indices");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of Indices tag != 1");
        }
        nl2 = ((Element)nl.item(0)).getElementsByTagName("index");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name = node.getAttributes().getNamedItem("name").getNodeValue();
        	String value = node.getAttributes().getNamedItem("value").getNodeValue();
        	
        	listIndexName.add(name);
        	listIndexFileName.add(value);
        }
        
        nl = doc.getElementsByTagName("Volatility");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of Volatility tag != 1");
        }
        
        mapVol = new HashMap<String, double[]>();
        nl2 = ((Element)nl.item(0)).getElementsByTagName("index");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name = node.getAttributes().getNamedItem("name").getNodeValue();
        	String value1 = node.getAttributes().getNamedItem("value1").getNodeValue();
        	String value2 = node.getAttributes().getNamedItem("value2").getNodeValue();
        	
        	double[] vTmp = new double[2];
        	vTmp[0] = Double.parseDouble(value1);
        	vTmp[1] = Double.parseDouble(value2);
        	mapVol.put(name.toLowerCase(), vTmp);
        }
        
        nl = doc.getElementsByTagName("GeometricMean");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of GeometricMean tag != 1");
        }
        
        mapMu = new HashMap<String, double[]>();
        nl2 = ((Element)nl.item(0)).getElementsByTagName("index");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name = node.getAttributes().getNamedItem("name").getNodeValue();
        	String value1 = node.getAttributes().getNamedItem("value1").getNodeValue();
        	String value2 = node.getAttributes().getNamedItem("value2").getNodeValue();
        	
        	double[] vTmp = new double[2];
        	vTmp[0] = Double.parseDouble(value1);
        	vTmp[1] = Double.parseDouble(value2);
        	mapMu.put(name.toLowerCase(), vTmp);
        }
        
        nl = doc.getElementsByTagName("Correlation");
        if(nl.getLength() != 1) {
        	throw new Exception("Number of Correlation tag != 1");
        }
        
        mapCor = new HashMap<String, Double>();
        nl2 = ((Element)nl.item(0)).getElementsByTagName("pair");
        for(int i=0; i<nl2.getLength(); ++i) {
        	Node node = nl2.item(i);
        	String name1 = node.getAttributes().getNamedItem("index1").getNodeValue();
        	String name2 = node.getAttributes().getNamedItem("index2").getNodeValue();
        	String value = node.getAttributes().getNamedItem("value").getNodeValue();
        	
        	mapCor.put(String.format("%s:%s", name1.toLowerCase(), name2.toLowerCase()), 
        			Double.parseDouble(value));
        }
        
	}
		
	protected double getVol(String name, int reg) throws Exception {
		String key = name.toLowerCase();
		if(mapVol.containsKey(key)) {
			return mapVol.get(key)[reg];
		} else {
			throw new Exception("Cannot find volatility: " + name);
		}
	}
	
	protected double getMu(String name, int reg) throws Exception {
		String key = name.toLowerCase();
		if(mapMu.containsKey(key)) {
			return mapMu.get(key)[reg];
		} else {
			throw new Exception("Cannot find geometric mean: " + name);
		}
	}
	
	protected double getCor(String name1, String name2) throws Exception {
		String key1 = String.format("%s:%s", name1.toLowerCase(), name2.toLowerCase());
		String key2 = String.format("%s:%s", name2.toLowerCase(), name1.toLowerCase());
		if(mapCor.containsKey(key1)) {
			return mapCor.get(key1);
		} else if(mapCor.containsKey(key2)) {
			return mapCor.get(key2);
		} else {
			throw new Exception("Cannot find volatility: " + name1 + ":" + name2);
		}
	}
	
	protected int getIntParam(String name) throws Exception {
		String key = name.toLowerCase();
		if(mapParameters.containsKey(key)) {
			return Integer.parseInt(mapParameters.get(key));
		} else {
			throw new Exception("Cannot find parameter: " + name);
		}
	}
	
	protected double getDoubleParam(String name) throws Exception {
		String key = name.toLowerCase();
		if(mapParameters.containsKey(key)) {
			return Double.parseDouble(mapParameters.get(key));
		} else {
			throw new Exception("Cannot find parameter: " + name);
		}
	}
	
	
	protected String getParam(String name) throws Exception {
		String key = name.toLowerCase();
		if(mapParameters.containsKey(key)) {
			return mapParameters.get(key);
		} else {
			throw new Exception("Cannot find parameter: " + name);
		}
	}
}
