package va.util;

import java.io.*;
import java.util.*;

import org.apache.log4j.Logger;


public class FmvConsolidator {
	private String seriatimFile;
	private String aggregateFile;
	private String folder;
	private String newline = System.getProperty("line.separator");
	protected Logger log = Logger.getLogger(FmvConsolidator.class);
	
	public FmvConsolidator(String folder) {
		this.folder = folder;
	}
	
	public void setSeriatimFile(String seriatimFile) {
		this.seriatimFile = seriatimFile;
	}
	
	public void setAggregateFile(String aggregateFile) {
		this.aggregateFile = aggregateFile;
	}
	
	public void consolidateSeriatim() throws Exception {
		String prefix = seriatimFile.replaceFirst("[.][^.]+$", "");
		File dir = new File(folder);
		File[] files = dir.listFiles();
		List<File> listFile = new ArrayList<File>();
		for (int i = 0; i < files.length; i++) {
		      if (files[i].isFile() && !files[i].getName().equals(seriatimFile) && files[i].getName().startsWith(prefix)) {
		    	  listFile.add(files[i]);
		      }
		}
		
		FileWriter outFile = new FileWriter(seriatimFile);
		PrintWriter out = new PrintWriter(outFile);
		
		int count = 0;
		
		for(File file : listFile) {
			StringBuilder sb = new StringBuilder();
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line = br.readLine();
			if(count == 0 ) {
				sb.append(line).append(newline);
			} 
			while(true) {
				line = br.readLine();
				if(line==null  || line.trim().equals("")) {
					break;
				}
				sb.append(line).append(newline);
			}
			++count;
			br.close();
			out.write(sb.toString());				
		}		
		out.close();
				
	}
	
	public void consolidateSeriatimT() throws Exception {
		String prefix = seriatimFile.replaceFirst("[.][^.]+$", "");
		File dir = new File(folder);
		File[] files = dir.listFiles();
		List<File> listFile = new ArrayList<File>();
		for (int i = 0; i < files.length; i++) {
		      if (files[i].isFile() && !files[i].getName().equals(seriatimFile) && files[i].getName().startsWith(prefix)) {
		    	  listFile.add(files[i]);
		      }
		}		
		
		String[] cell;
		List<String> listPVName = new ArrayList<String>();
		for(File file : listFile) {
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line = br.readLine();
			cell = line.split(",");
			for(int i=3; i<cell.length; ++i) {				
				listPVName.add(cell[i]);
			}
			br.close();
			break;
		}
							
		Map<String, Map<String, List<String>>> mapContent = 
				new HashMap<String, Map<String, List<String>>>();
		for(File file : listFile) {
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line = br.readLine();
			
			while(true) {
				line = br.readLine();
				if(line==null  || line.trim().equals("")) {
					break;
				}
				cell = line.split(",");
				String key = cell[0] + ":" + cell[1];
				if(!mapContent.containsKey(cell[2])) {
					mapContent.put(cell[2], new HashMap<String, List<String>>());
				}
				List<String> listPV = new ArrayList<String>();
				for(int j=3; j<cell.length; ++j) {
					listPV.add(cell[j]);
				}
				mapContent.get(cell[2]).put(key, listPV);	
			}
			
			br.close();
						
		}				
				
		if(mapContent.size() ==0) {
			return;
		}
		
		for(int k=0; k<listPVName.size(); ++k) {
			FileWriter writer = new FileWriter(listPVName.get(k) + "_seriatim.csv");
			List<String> listHeader = new ArrayList<String>();
			
			int count = 0;
			for(String key1 : mapContent.keySet()) {
				if(count==0) {
					for(String key2 : mapContent.get(key1).keySet()) {
						listHeader.add(key2);
					}
					writer.write("recordID");
					for(int i=0; i<listHeader.size(); ++i) {
						writer.write(",");
						writer.write(listHeader.get(i));
					}
					writer.write(newline);
				} 
				count++;
				writer.write(key1);
						
				for(int i=0; i<listHeader.size(); ++i) {
					writer.write(",");
					writer.write(mapContent.get(key1).get(listHeader.get(i)).get(k));
				}
				writer.write(newline);
							
			}
				
			writer.flush();
		    writer.close();
		}
	}
	
	public void consolidateAggregate() throws Exception {
		String prefix = aggregateFile.replaceFirst("[.][^.]+$", "");
		File dir = new File(folder);
		File[] files = dir.listFiles();
		List<File> listFile = new ArrayList<File>();
		for (int i = 0; i < files.length; i++) {
		      if (files[i].isFile() && !files[i].getName().equals(aggregateFile) && files[i].getName().startsWith(prefix)) {
		    	  listFile.add(files[i]);		    	  
		      }
		}
		
		
		StringBuilder sb = new StringBuilder();
		Map<String, Map<String, double[]>> mmData = new HashMap<String, Map<String, double[]>>();
		int count = 0;		
		String[] cell;
		double[] vTemp;
		for(File file : listFile) {		
			log.info(file.getName());
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line = br.readLine();
			int nCol = line.split(",").length - 3;
			if(count == 0) {
				sb.append(line).append(newline);
			} 
			while(true) {
				line = br.readLine();
				if(line==null || line.trim().equals("")) {
					break;
				}
				cell = line.split(",");				
						
				vTemp = new double[nCol];		
				for(int j=0; j<nCol; ++j) {
					vTemp[j] = Double.parseDouble(cell[3+j]);
				}
				if(mmData.containsKey(cell[0])) {
					if(mmData.get(cell[0]).containsKey(cell[1])) {	
						for(int j=0; j<nCol; ++j) {							
						    vTemp[j] +=  mmData.get(cell[0]).get(cell[1])[j];	
						}
						mmData.get(cell[0]).put(cell[1], vTemp);
					} else {
						mmData.get(cell[0]).put(cell[1], vTemp);
					}
				} else {							
					mmData.put(cell[0], new HashMap<String, double[]>());
				}
			}
			++count;
			br.close();					
		}		
		
		for(String key1 : mmData.keySet()) {
			for(String key2 : mmData.get(key1).keySet()) {
				vTemp = mmData.get(key1).get(key2);
				sb.append(String.format("%s,%s,0", key1, key2));
				for(int j=0; j<vTemp.length; ++j) {
					sb.append(String.format(",%.6f",vTemp[j])); 
				}
				sb.append(newline);
			}
		}
		
		FileWriter outFile = new FileWriter(aggregateFile);
		PrintWriter out = new PrintWriter(outFile);
		out.write(sb.toString());			
		out.close();
				
	}
}
