package va.util;

import java.io.*;
import java.util.*;

public class GreekCalculator {
	protected String newline = System.getProperty("line.separator");
	protected String mapFile;
	protected String fmvFile;
	protected String outputFile;
	
	protected List<String[]> mGreekName;
	protected Map<String, List<Double>> mapFMV;
	protected double[][] mGreek;
	protected int nRecord;
	
	public GreekCalculator(String mapFile, String fmvFile, String outputFile) {
		this.mapFile = mapFile;
		this.fmvFile = fmvFile;
		this.outputFile = outputFile;
	}
	
	public void work() throws Exception {
		readMapFile();
		readFmvFile();
		
		for(String s : mapFMV.keySet()) {
			nRecord = mapFMV.get(s).size();
			break;
		}
		
		mGreek = new double[nRecord][mGreekName.size()];
		for(int j=0; j<mGreekName.size(); ++j) {
			String[] vName = mGreekName.get(j);
			double dShock = Double.parseDouble(vName[3]);
			for(int i=0; i<nRecord; ++i) {
				mGreek[i][j] = (mapFMV.get(vName[1]).get(i) - mapFMV.get(vName[2]).get(i)) / dShock;
			}
		}
		
		
		save();
	}
	
	protected void readMapFile() throws Exception {
		mGreekName = new ArrayList<String[]>();
		BufferedReader br = new BufferedReader(new FileReader(mapFile));
		String line = br.readLine();
		String[] cell;
		while(true) {
			line = br.readLine();
			if(line==null  || line.trim().equals("")) {
				break;
			}
			cell = line.split(",");
			mGreekName.add(cell);			
		}
		br.close();
	}
	
	protected void readFmvFile() throws Exception {
		mapFMV = new HashMap<String, List<Double>>();
				
		BufferedReader br = new BufferedReader(new FileReader(fmvFile));
		String line = br.readLine();
		String[] header = line.split(",");
		for(String key : header) {
			mapFMV.put(key, new ArrayList<Double>());
		}
		String[] cell;
		while(true) {
			line = br.readLine();
			if(line==null  || line.trim().equals("")) {
				break;
			}
			cell = line.split(",");
			for(int i=0; i<cell.length; ++i) {
			    mapFMV.get(header[i]).add(Double.parseDouble(cell[i]));
			}
		}
		br.close();
	}
	
	protected void save() throws Exception {
		FileWriter writer = new FileWriter(outputFile);
				
		writer.write("recordID");
		if(mapFMV.containsKey("base:base")) {
			writer.write(",fmv");
		}
		for(int j=0; j<mGreekName.size(); ++j) {
			writer.write(",");
			writer.write(mGreekName.get(j)[0]);
		}
		writer.write(newline);
		for(int i=0; i<nRecord; ++i) {
			writer.write(String.format("%.0f", mapFMV.get("recordID").get(i)));
			if(mapFMV.containsKey("base:base")) {
				writer.write(String.format(",%.6f", mapFMV.get("base:base").get(i)));
			}
			for(int j=0; j<mGreekName.size(); ++j) {
				writer.write(String.format(",%.6f", mGreek[i][j]));
			}
			writer.write(newline);
		}
		
		writer.flush();
	    writer.close();
	}
}
